/*
Copyright (C) 2010

This file is part of the Hough applet
written by Max Bügler
http://www.maxbuegler.eu/

Hough applet is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Hough applet is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package HoughTransform;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

/**
 * Date: May 1, 2010
 * Time: 10:09:57 AM
 * <p/>
 * Copyright 2010 by Max Buegler.
 * Licensed under General Public License Version 3
 */

public class BinaryPaintComponent extends JPanel {
    private boolean[][] image;
    private boolean paintable;
    private int stroke;
    private ArrayList<ActionListener> listener;

    public BinaryPaintComponent(boolean[][] image, boolean paintable) {
        this.image = image;
        this.paintable = paintable;
        this.stroke=2;
        addMouseListener(new Listener());
        addMouseMotionListener(new Listener());
        listener=new ArrayList<ActionListener>();
    }

    public BinaryPaintComponent() {
        this.image=null;
        this.stroke=2;
        this.paintable=true;
        addMouseListener(new Listener());
        addMouseMotionListener(new Listener());
        listener=new ArrayList<ActionListener>();
    }

    public void addActionListener(ActionListener a){
        listener.add(a);
    }

    public boolean isPaintable() {
        return paintable;
    }

    public void setPaintable(boolean paintable) {
        this.paintable = paintable;
    }

    public void setImage(boolean[][] i){
        this.image=i;
        repaint();
    }

    public boolean[][] getImage() {
        return image;
    }

    public int getStroke() {
        return stroke;
    }

    public void setStroke(int stroke) {
        this.stroke = stroke;
    }

    public void clear(){
        image=new boolean[getWidth()][getHeight()];
        for (ActionListener l:listener)l.actionPerformed(null);
        repaint();
    }
    public void paintComponent(Graphics g){
        if (paintable){
            if (image==null||getWidth()!=image.length||getHeight()!=image[0].length){
                if (image!=null){
                    boolean[][] nimage=new boolean[getWidth()][getHeight()];
                    for (int x=0;x<getWidth()&&x<image.length;x++){
                        for (int y=0;y<getHeight()&&y<image[0].length;y++){
                            nimage[x][y]=image[x][y];
                        }
                    }
                    image=nimage;
                }
                else
                    image=new boolean[getWidth()][getHeight()];
            }
        }
        if (image!=null){
            for (int x=0;x<image.length;x++){
                for (int y=0;y<image[0].length;y++){
                    if (image[x][y])g.setColor(Color.BLACK);
                    else g.setColor(Color.WHITE);
                    g.drawRect(x,y,1,1);
                }
            }
        }
        else{
            g.setColor(Color.WHITE);
            g.fillRect(0,0,getWidth(),getHeight());
        }
        g.setColor(Color.BLUE);
        g.drawRect(1,1,getWidth()-2,getHeight()-2);
    }

    private void paint(int xc, int yc){
        for (int x=Math.max(0,xc-stroke);x<Math.min(xc+stroke,image.length-1);x++){
            for (int y=Math.max(0,yc-stroke);y<Math.min(yc+stroke,image[0].length-1);y++){
                double d=Math.sqrt(Math.pow(xc-x,2)+Math.pow(yc-y,2));
                if (d<=stroke)image[x][y]=true;
            }
        }

        repaint();
    }

    class Listener implements MouseListener, MouseMotionListener {
        private int lx,ly;
        private boolean down=false;
        public void mouseClicked(MouseEvent e) {
            if (!paintable)return;
            paint(e.getX(),e.getY());
            for (ActionListener l:listener)l.actionPerformed(null);
        }

        public void mouseDragged(MouseEvent e) {
            if (!paintable)return;
            if (down){
                int x2=e.getX();
                int y2=e.getY();
                int dx=e.getX()-lx;
                int dy=e.getY()-ly;

                if (Math.abs(dx)>Math.abs(dy)){
                    double slope=(double)dy/dx;
                    for (int x=lx;x!=e.getX();x+=Math.signum(dx)){
                        int y=(int)Math.round(ly+slope*(x-lx));
                        paint(x,y);
                    }
                }
                else{
                    double slope=(double)dx/dy;
                    for (int y=ly;y!=e.getY();y+=Math.signum(dy)){
                        int x=(int)Math.round(lx+slope*(y-ly));
                        paint(x,y);
                    }
                }
            }
            lx=e.getX();
            ly=e.getY();
            paint(e.getX(),e.getY());
            for (ActionListener l:listener)l.actionPerformed(null);
            down=true;
        }
        public void mousePressed(MouseEvent e) {}
        public void mouseMoved(MouseEvent e) {
            if (e.getButton()!=MouseEvent.BUTTON1)down=false;
        }
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
    }
    public static void main(String[] args){
        JFrame f=new JFrame("FUN");
        BinaryPaintComponent bpc=new BinaryPaintComponent();
        f.add(bpc);
        f.setSize(320,240);
        f.setVisible(true);
    }
}
