/*
Copyright (C) 2010

This file is part of the Hough applet
written by Max Bügler
http://www.maxbuegler.eu/

Hough applet is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Hough applet is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package HoughTransform;

import java.awt.image.BufferedImage;

/**
 * Date: May 1, 2010
 * Time: 12:31:07 PM
 * <p/>
 * Copyright 2010 by Max Buegler.
 * Licensed under General Public License Version 3
 */

public class HoughTransform {
    public static BufferedImage lineHoughTransform(boolean[][] in, int theta,int rad, boolean inv){
        int[][] output=new int[rad][theta];
        double anglemultiplier=Math.PI/theta;
        double maxrad=Math.sqrt(Math.pow(in.length,2)+Math.pow(in[0].length,2));
        double radmultiplier=rad/(2*(maxrad+10));
        int maxval=0;
        for (int x=0;x<in.length;x++){
            for (int y=0;y<in[0].length;y++){
                if (in[x][y]==inv){
                    for (int t=0;t<theta;t++){
                        double omega=t*anglemultiplier;
                        int radius=(int)Math.round(radmultiplier*(maxrad+x*Math.cos(omega)+y*Math.sin(omega)));
                        output[radius][t]++;
                        if (output[radius][t]>maxval)maxval=output[radius][t];
                    }
                }
            }
        }

        BufferedImage outimg=new BufferedImage(theta,rad,BufferedImage.TYPE_BYTE_GRAY);
        for (int x=0;x<rad;x++){
            for (int y=0;y<theta;y++){
                int v=255-(int)Math.round((255.0*output[x][y])/maxval);
                int c=
                ((v & 0xFF) << 16) |
                ((v & 0xFF) << 8)  |
                ((v & 0xFF) << 0);
                outimg.setRGB(y,x,c);
            }
        }
        return outimg;
    }

    public static BufferedImage cornerHoughTransform(boolean[][] in, int xout,int yout, int dist, boolean inv){
        int[][] output=new int[xout][yout];
        double xmultiplier=(double)in.length/xout;
        double ymultiplier=(double)in[0].length/yout;
        int maxval=0;
        for (int x=0;x<in.length;x++){
            for (int y=0;y<in[0].length;y++){
                if (in[x][y]==inv){
                    for (int xc=Math.max(0,x-dist);xc<Math.min(in.length-1,x+dist);xc++){
                        int ox=(int)Math.round(xmultiplier*xc);
                        int oy=(int)Math.round(ymultiplier*y);
                        output[ox][oy]++;
                        if (output[ox][oy]>maxval)maxval=output[ox][oy];
                    }
                    for (int yc=Math.max(0,y-dist);yc<Math.min(in[0].length-1,y+dist);yc++){
                        int ox=(int)Math.round(xmultiplier*x);
                        int oy=(int)Math.round(ymultiplier*yc);
                        output[ox][oy]++;
                        if (output[ox][oy]>maxval)maxval=output[ox][oy];
                    }

                }
            }
        }

        BufferedImage outimg=new BufferedImage(xout,yout,BufferedImage.TYPE_BYTE_GRAY);
        for (int x=0;x<xout;x++){
            for (int y=0;y<yout;y++){
                int v=255-(int)Math.round(255*Math.pow(((double)output[x][y]/maxval),8));
                int c=
                ((v & 0xFF) << 16) |
                ((v & 0xFF) << 8)  |
                ((v & 0xFF) << 0);
                outimg.setRGB(x,y,c);
            }
        }
        return outimg;
    }

}
